﻿
using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

/*                     Digital image filtering               */
/*                      www.mosismath.com                    */

namespace WindowsFormsApplication1
{

   
    public partial class MainWin : Form
    {

        Bitmap bmpSource;
        Bitmap bmpDest;
        double[,] s;

        public MainWin()
        {
            InitializeComponent();
            byte[] buffer;
            double[,] s = new double[3, 3];

            // Read image file
            FileStream fileStream = new FileStream(Application.StartupPath + "\\MOND.BMP", FileMode.Open, FileAccess.Read);

            try
            {
                int length = (int)fileStream.Length;  
                buffer = new byte[length];           
                int count;                         
                int sum = 0;                         

                while ((count = fileStream.Read(buffer, sum, length - sum)) > 0)
                    sum += count; 
            }
            finally
            {
                fileStream.Close();
            }
            MemoryStream imageStream = new MemoryStream(buffer, 0, buffer.Length);
            string tempString = System.Text.Encoding.ASCII.GetString(buffer);
            char[] bufferJpeg = new char[tempString.Length];
            tempString.CopyTo(0, bufferJpeg, 0, tempString.Length);
            imageStream.Position = 0;
            Bitmap bmpSource = new Bitmap(imageStream);
            Bitmap bmpDest = new Bitmap(pictureBox2.Image);
            cbFilterSelect.SelectedIndex = 0;
            cbFilterSelect_DropDownClosed(cbFilterSelect, new EventArgs());
        }


        private Bitmap ProcessImage(Bitmap bmpSource, double[,] s, int order)
        {
            int x, y, i, j;
            double R, G, B;
            Bitmap bmpDest = new Bitmap(pictureBox2.Image);
            using (Graphics g = Graphics.FromImage(bmpSource))
            {
                Pen bluePen = new Pen(Color.Blue, 2);
                Pen redPen = new Pen(Color.Red, 4);
                for (x = 1; x < bmpSource.Width - 1; x++)
                {
                    for (y = 1; y < bmpSource.Height - 1; y++)
                    {
                        R = 0;
                        G = 0;
                        B = 0;
                        for (i = 0; i < order; i++)
                        {
                            for (j = 0; j < order; j++)
                            {
                                Color pix = bmpSource.GetPixel(x - 1 + i, y - 1 + j);
                                R += (int)(pix.R * s[i, j]);
                                G += (int)(pix.G * s[i, j]);
                                B += (int)(pix.B * s[i, j]);
                            }
                        }
                        if (R < 0)
                            R = 0;
                        else
                        {
                            if (R > 255)
                                R = 255;
                        }
                        if (G < 0)
                            G = 0;
                        else
                        {
                            if (G > 255)
                                G = 255;
                        }
                        if (B < 0)
                            B = 0;
                        else
                        {
                            if (B > 255)
                                B = 255;
                        }
                        Color newPix = Color.FromArgb((int)(R), (int)(G), (int)(B));
                        bmpDest.SetPixel(x, y, newPix);
                    }
                }
                g.Dispose();
            }
            return bmpDest;
        }


        private void button1_Click(object sender, EventArgs e)
        {
            bmpSource = new Bitmap(pbBild.Image);
            bmpDest = ProcessImage(bmpSource, s, 3);
            pictureBox2.Image = bmpDest;
            pictureBox2.Image.Save(Application.StartupPath + "\\Mond_out.jpg");
        }


        private void cbFilterSelect_DropDownClosed(object sender, EventArgs e)
        {
            double[,] smoothing = { { 1.0 / 9.0, 1.0 / 9.0, 1.0 / 9.0 }, { 1.0 / 9.0, 1.0 / 9.0, 1.0 / 9.0 }, { 1.0 / 9.0, 1.0 / 9.0, 1.0 / 9.0 } };

            // Gauss low pass
            double[,] gauss = { { 1.0 / 16.0, 2.0 / 16.0, 1.0 / 16.0 }, { 2.0 / 16.0, 4.0 / 16.0, 2.0 / 16.0 }, { 1.0 / 16.0, 2.0 / 16.0, 1.0 / 16.0 } };

            // Laplace
            double[,] laplace_1 = { { 0, -1.0, 0 }, { -1.0, 5.0, -1.0 }, { 0, -1.0, 0 } };
            double[,] laplace_2 = { { -0.5, -1.0, -0.5 }, { -1.0, 7.0, -1.0 }, { -0.5, -1.0, -0.5 } };
            double[,] laplacenoise = { { -1.0, -1.0, -1.0 }, { -1.0, 9.0, -1.0 }, { -1.0, -1.0, -1.0 } };

            switch(cbFilterSelect.SelectedIndex)
            {
                case 0:
                    s = smoothing;
                break;
                case 1:
                    s = gauss;
                    break;
                case 2:
                    s = laplace_1;
                    break;
                case 3:
                    s = laplacenoise;
                    break;
                case 4:
                    s = laplace_2;
                    break;
            }          
        }
    }
}
